<?php
if(!defined('_CLASS_FTPFOLDER')) {
   define('_CLASS_FTPFOLDER', '_CLASS_FTPFOLDER');

class FtpFolder{
	var $home_directory			= './';
	var $ftp_home_directory		= './';
	var $ftpsize_file			= 'ftpsize.dat';

	function folder_is_safe($folder) {
		if ($folder == '' || $folder == '.' || $folder == '..')
			return false;

		$tempfolder = str_replace('\\', '/', $folder);
		if ($tempfolder[0] != '/')
			$tempfolder = '/'.$tempfolder;
		if (substr($tempfolder, -1) != '/')
			$tempfolder = $tempfolder.'/';

		if (strpos($tempfolder, '/./') !== false || strpos($tempfolder, '/../') !== false)
			return false;

		$len = strlen($folder);
		
		$strTemp = ':*?"<>|';
		for ($i = 0; $i < $len; $i++) {
			$ch = ord($folder[$i]);
			if ($ch < 0x20) 
				return false;

			if (strpos($strTemp, $folder[$i]) !== false)
				return false;
		}
		
		return true;
	}

	function get_folder_tree($ftpfolder) {
		$folders = array();
		
		$temppath = $this->ftp_home_directory.$ftpfolder;
		
		$dh = opendir($temppath);
		if ($dh){
			while($file = readdir($dh)) {
				if ($file == '.' || $file == '..')
					continue;
					
				$fullfile = $temppath.'/'.$file;
				if (is_dir($fullfile)) {
					$path = $ftpfolder.'/'.$file;
	
					$item = array();
					$item['id'] = base64_encode($file);
					$item['name'] = $files;
					$item['folder'] = $path;
					$item['childs'] = $this->get_folder_tree($path);
					
					$folders[] = $item;
				}						
			}

			closedir($dh);
		}
		
		return $folders;
	}

	function get_all_folder($ftpfolder) {
		$folders = array();
		
		$foldername = ($ftpfolder == '') ? '/' : $ftpfolder;

		$item = array();
		$item['id'] = base64_encode($foldername);
		$item['name'] = $foldername;
		$folders[] = $item;
	
		$temppath = $this->ftp_home_directory.$ftpfolder;
		
		$dh = opendir($temppath);
		if ($dh){
			while($file = readdir($dh)) {
				if ($file == '.' || $file == '..')
					continue;
				
				$fullfile = $temppath.'/'.$file;
				if (is_dir($fullfile)) {
					$subfolders = $this->get_all_folder($ftpfolder.'/'.$file);
					
					foreach($subfolders as $item)
						$folders[] = $item;
				}						
			}

			closedir($dh);
		}
	
		return $folders;
	}

	function get_file_list($ftpfolder, &$folderList, &$fileList, &$folderInfo) {
		$folder_count = 0;
		$file_count = 0;
		$total_size = 0;
	
		$temppath = $this->ftp_home_directory.$ftpfolder;
		
		$dh = opendir($temppath);
		if ($dh){
			while($file = readdir($dh)) {
				if ($file == '.' || $file == '..')
					continue;
				
				$fullfile = $temppath.'/'.$file;
				
				$info['id'] = base64_encode($file);
				$info['name'] = $file;
				$info['size'] = filesize($fullfile);
				$info['date'] = filemtime($fullfile);
	
				if (is_dir($fullfile)) {
					$info['isdir'] = true;

					$folderList[] = $info;
					$folder_count++;
				}
				else {
					$info['isdir'] = false;
					
					$fileList[] = $info;
					$file_count++;
				}
				$total_size += $info['size'];
			}
			
			closedir($dh);
		}
	
		$folderInfo['foldercount'] = $folder_count;
		$folderInfo['filecount'] = $file_count;
		$folderInfo['totalused'] = $total_size;
	}

	function is_file($ftpfolder) {
		$folderinfo = array();

		$temppath = $this->ftp_home_directory.$ftpfolder;
		if (!file_exists($temppath))
			return false;

		if (is_file($temppath)) 
			return true;

		return false;
	}

	function get_folder_info($ftpfolder) {
		$folderinfo = array();

		$temppath = $this->ftp_home_directory.$ftpfolder;
		if (!file_exists($temppath))
			return false;

		if (is_dir($temppath)) {
	    	$folderinfo['folder'] = 1;
	    	$folderinfo['file'] = 0;
	    	$folderinfo['size'] = 0;

			$dh = opendir($temppath);
			if ($dh){
				while($file = readdir($dh)) {
					if ($file == '.' || $file == '..')
						continue;
					
					$fullfolder = $ftpfolder.'/'.$file;
					$subfolderinfo = $this->get_folder_info($fullfolder);
					
					if ($subfolderinfo != false) {
				    	$folderinfo['folder'] += $subfolderinfo['folder'];
				    	$folderinfo['file'] += $subfolderinfo['file'];
				    	$folderinfo['size'] += $subfolderinfo['size'];
				    }
				}
			}
			
			closedir($dh);
		}
		else {
	    	$folderinfo['folder'] = 0;
	    	$folderinfo['file'] = 1;
	    	$folderinfo['size'] = filesize($temppath);
		}
		
		return $folderinfo;
	}
	
	function get_folder_file($ftpfolder) {
		$filelist = array();

		$temppath = $this->ftp_home_directory.$ftpfolder;
		if (!file_exists($temppath))
			return false;

		if (is_dir($temppath)) {
			$dh = opendir($temppath);
			if ($dh){
				while($file = readdir($dh)) {
					if ($file == '.' || $file == '..')
						continue;
					
					$fullfolder = $ftpfolder.'/'.$file;
					$subfolderinfo = $this->get_folder_file($fullfolder);
						
					if ($subfolderinfo != false) {
			    		foreach ($subfolderinfo as $item)
		    				$filelist[] = $item;
		    		}
				}
			}
			
			closedir($dh);
		}
		else {
			$info = array();
			
	    	$info['name'] = $ftpfolder;
	    	$info['date'] = filemtime($temppath);
	    	$info['size'] = filesize($temppath);
	    	
	    	$filelist[] = $info;
		}
		
		return $filelist;
	}
	
	function create_folder($newfolder) {
		if (!$this->folder_is_safe($newfolder))
			return false;
		
		$newpath = $this->ftp_home_directory.$newfolder;
		if (file_exists($newpath))
			return false;

		return @mkdir($newpath);	
	}

	function modify_folder_file($oldfolder, $newfolder) {
		if (!$this->folder_is_safe($oldfolder) || !$this->folder_is_safe($newfolder))
			return false;
		
		if (strcasecmp($oldfolder, $newfolder) == 0)
			return false;
		
		$oldpath = $this->ftp_home_directory.$oldfolder;
		$newpath = $this->ftp_home_directory.$newfolder;

		if (file_exists($newpath))
			return false;

		return @rename($oldpath, $newpath);	
	}

	function delete_folder_file($ftpfolder) {
		if (!$this->folder_is_safe($ftpfolder))
			return false;

		$info = array();
		$info['count'] = 0;
		$info['size'] = 0;
		
		$temppath = $this->ftp_home_directory.$ftpfolder;
		if (is_dir($temppath)) {
			$dh = opendir($temppath);
			if ($dh){
				while($file = readdir($dh)) {
					if ($file == '.' || $file == '..')
						continue;
					
					$fullfolder = $ftpfolder.'/'.$file;
					$subinfo = $this->delete_folder_file($fullfolder);
					if ($subinfo != false) {
						$info['count'] += $subinfo['count'];
						$info['size'] += $subinfo['size'];
		    		}
				}
				
				closedir($dh);
			}

		    rmdir($temppath);
		}
		else {
			$size = filesize($temppath);
			
			if (@unlink($temppath)) {
				$info['count'] = 1;
				$info['size'] = $size;
		    }
		}
		
		return $info;
	}
	
	function move_folder_file($oldfolder, $newfolder) {
		if (!$this->folder_is_safe($oldfolder) || !$this->folder_is_safe($newfolder))
			return false;

		if (strcasecmp($oldfolder, $newfolder) == 0)
			return false;

		$oldpath = $this->ftp_home_directory.$oldfolder;
		$newpath = $this->ftp_home_directory.$newfolder;

		return @rename($oldpath, $newpath);
	}
	
	function copy_folder_file($oldfolder, $newfolder) {
		if (!$this->folder_is_safe($oldfolder) || !$this->folder_is_safe($newfolder))
			return false;

		if (strcasecmp($oldfolder, $newfolder) == 0)
			return false;

		$info = array();
		$info['count'] = 0;
		$info['size'] = 0;

		$oldpath = $this->ftp_home_directory.$oldfolder;
		$newpath = $this->ftp_home_directory.$newfolder;
		
		if (is_dir($oldpath)) {
			if (!file_exists($newpath))
				@mkdir($newpath);
				
			$dh = opendir($oldpath);
			if ($dh){
				while($file = readdir($dh)) {
					if ($file == '.' || $file == '..')
						continue;
					
					$oldfullfolder = $oldfolder.'/'.$file;
					$newfullfolder = $newfolder.'/'.$file;
					
					$subinfo = $this->copy_folder_file($oldfullfolder, $newfullfolder);
					if ($subinfo != false) {
						$info['count'] += $subinfo['count'];
						$info['size'] += $subinfo['size'];
					}
				}
				
				closedir($dh);
			}
		}
		else {
			if (@copy($oldpath, $newpath)) {
				$info['count'] = 1;
				$info['size'] = filesize($newpath);
			}
		}
		
		return $info;
	}

	function get_ftp_quota() {
		$sizefile = $this->home_directory.$this->ftpsize_file;
	
		$ftpquota = 0;
		$ftpcount = 0;
		$lines = file($sizefile);
		
		$linecount = count($lines);
		for($i = 0; $i < $linecount; $i++)	{
			$tmp = trim($lines[$i]);
			$pos = strpos($tmp, ' ');
			if($pos === false)
				continue;
				
			$pos_S = strpos($tmp, 'S');
			$pos_C = strpos($tmp, 'C');
						
			if( !($pos_S === false)
				&& !($pos_C === false)) {
				$ftpquota = substr($tmp, 0, $pos_S);
				$ftpcount = substr($tmp, $pos+1, $pos_C - $pos -1);
				break;
			}
		}
		
		if($i >= $linecount)
			return true;
		
		if($ftpquota <= 0 ) $ftpquota = 0;
		if($ftpcount <= 0 ) $ftpcount = 0;
		
		$usedftpquota = 0;
		$usedftpcount = 0;
		
		$i++;
		$linecount = count($lines);
		for(; $i < $linecount; $i++) {
			$tmp = trim($lines[$i]);
			$pos = strpos($tmp, ' ');
			if($pos === false)
				continue;
			
			$usedftpquota += substr($tmp, 0, $pos);
			$usedftpcount += substr($tmp, $pos+1);			
		}
		
		if ($usedftpquota < 0)
			$usedftpquota = 0;
		if ($usedftpcount < 0)
			$usedftpcount = 0;
		
		$info = array();
		$info['ftpquota'] = $ftpquota;
		$info['ftpcount'] = $ftpcount;
		$info['usedftpquota'] = $usedftpquota;
		$info['usedftpcount'] = $usedftpcount;
		
		return $info;
	}
	
	function set_ftp_quota($size, $count, $flag) {
		$strLine = '';
		$sizefile = $this->home_directory.$this->ftpsize_file;
		
		if ($flag != '+' && $flag != '-')
			return;
		
		if ($size == 0 && $count == 0)
			return;

		if (file_exists($sizefile) == false || filesize($sizefile) == 0)
			$strLine .= '0S 0C'."\r\n";
		
		if ($flag == '+')
			$strLine .= floatval($size).' '.intval($count)."\r\n";
		else
			$strLine .= '-'.floatval($size).' -'.intval($count)."\r\n";			
			
		$fp = fopen($sizefile, 'ab');
		if ($fp) {
			fwrite($fp, $strLine);
			fclose($fp);
		}
	}
}

}

?>